@@ -3,7 +3,7 @@ package List::AllUtils;
 use strict;
 use warnings;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 use List::Util 1.45      ();
 use List::SomeUtils 0.56 ();
@@ -50,7 +50,7 @@ List::AllUtils - Combines List::Util, List::SomeUtils and List::UtilsBy in one b
 
 =head1 VERSION
 
-version 0.15
+version 0.16
 
 =head1 SYNOPSIS
 
@@ -65,10 +65,10 @@ version 0.15
 
 =head1 DESCRIPTION
 
-Are you sick of trying to remember whether a particular helper is
-defined in L<List::Util>,  L<List::SomeUtils> or L<List::UtilsBy>? I sure am. Now you
-don't have to remember. This module will export all of the functions
-that either of those three modules defines.
+Are you sick of trying to remember whether a particular helper is defined in
+L<List::Util>, L<List::SomeUtils> or L<List::UtilsBy>? I sure am. Now you
+don't have to remember. This module will export all of the functions that
+either of those three modules defines.
 
 Note that all function documentation has been shamelessly copied from
 L<List::Util>, L<List::SomeUtils> and L<List::UtilsBy>.
@@ -81,8 +81,8 @@ overlap with L<List::UtilsBy>. C<List::AllUtils> always favors the version
 provided by L<List::Util>, L<List::SomeUtils> or L<List::UtilsBy> in that
 order.
 
-The docs below come from L<List::Util> 1.31, L<List::SomeUtils> 0.50, and
-L<List::UtilsBy> 0.10.
+The docs below come from L<List::Util> 1.54, L<List::SomeUtils> 0.58, and
+L<List::UtilsBy> 0.11.
 
 =head1 WHAT IS EXPORTED?
 
@@ -93,225 +93,543 @@ your installed version provides.
 
 =head1 LIST-REDUCTION FUNCTIONS
 
-The following set of functions all reduce a list down to a single value.
+The following set of functions all apply a given block of code to a list of
+values.
+=head2 reduce
 
-=head2 reduce BLOCK LIST
+    $result = reduce { BLOCK } @list
 
-Reduces LIST by calling BLOCK, in a scalar context, multiple times,
-setting C<$a> and C<$b> each time. The first call will be with C<$a>
-and C<$b> set to the first two elements of the list, subsequent
-calls will be done by setting C<$a> to the result of the previous
-call and C<$b> to the next element in the list.
+Reduces C<@list> by calling C<BLOCK> in a scalar context multiple times,
+setting C<$a> and C<$b> each time. The first call will be with C<$a> and C<$b>
+set to the first two elements of the list, subsequent calls will be done by
+setting C<$a> to the result of the previous call and C<$b> to the next element
+in the list.
 
-Returns the result of the last call to BLOCK. If LIST is empty then
-C<undef> is returned. If LIST only contains one element then that
-element is returned and BLOCK is not executed.
+Returns the result of the last call to the C<BLOCK>. If C<@list> is empty then
+C<undef> is returned. If C<@list> only contains one element then that element
+is returned and C<BLOCK> is not executed.
 
+The following examples all demonstrate how C<reduce> could be used to implement
+the other list-reduction functions in this module. (They are not in fact
+implemented like this, but instead in a more efficient manner in individual C
+functions).
+
+    $foo = reduce { defined($a)            ? $a :
+                    $code->(local $_ = $b) ? $b :
+                                             undef } undef, @list # first
+
+    $foo = reduce { $a > $b ? $a : $b } 1..10       # max
+    $foo = reduce { $a gt $b ? $a : $b } 'A'..'Z'   # maxstr
     $foo = reduce { $a < $b ? $a : $b } 1..10       # min
     $foo = reduce { $a lt $b ? $a : $b } 'aa'..'zz' # minstr
     $foo = reduce { $a + $b } 1 .. 10               # sum
     $foo = reduce { $a . $b } @bar                  # concat
 
-If your algorithm requires that C<reduce> produce an identity value, then
-make sure that you always pass that identity value as the first argument to prevent
+    $foo = reduce { $a || $code->(local $_ = $b) } 0, @bar   # any
+    $foo = reduce { $a && $code->(local $_ = $b) } 1, @bar   # all
+    $foo = reduce { $a && !$code->(local $_ = $b) } 1, @bar  # none
+    $foo = reduce { $a || !$code->(local $_ = $b) } 0, @bar  # notall
+       # Note that these implementations do not fully short-circuit
+
+If your algorithm requires that C<reduce> produce an identity value, then make
+sure that you always pass that identity value as the first argument to prevent
 C<undef> being returned
 
   $foo = reduce { $a + $b } 0, @values;             # sum with 0 identity value
 
-The remaining list-reduction functions are all specialisations of this
-generic idea.
+The above example code blocks also suggest how to use C<reduce> to build a
+more efficient combined version of one of these basic functions and a C<map>
+block. For example, to find the total length of all the strings in a list,
+we could use
 
-=head2 first BLOCK LIST
+    $total = sum map { length } @strings;
 
-Similar to C<grep> in that it evaluates BLOCK setting C<$_> to each element
-of LIST in turn. C<first> returns the first element where the result from
-BLOCK is a true value. If BLOCK never returns true or LIST was empty then
-C<undef> is returned.
+However, this produces a list of temporary integer values as long as the
+original list of strings, only to reduce it down to a single value again. We
+can compute the same result more efficiently by using C<reduce> with a code
+block that accumulates lengths by writing this instead as:
+
+    $total = reduce { $a + length $b } 0, @strings
+
+The other scalar-returning list reduction functions are all specialisations of
+this generic idea.
+
+=head2 reductions
+
+    @results = reductions { BLOCK } @list
+
+I<Since version 1.54.>
+
+Similar to C<reduce> except that it also returns the intermediate values along
+with the final result. As before, C<$a> is set to the first element of the
+given list, and the C<BLOCK> is then called once for remaining item in the
+list set into C<$b>, with the result being captured for return as well as
+becoming the new value for C<$a>.
+
+The returned list will begin with the initial value for C<$a>, followed by
+each return value from the block in order. The final value of the result will
+be identical to what the C<reduce> function would have returned given the same
+block and list.
+
+    reduce     { "$a-$b" }  "a".."d"    # "a-b-c-d"
+    reductions { "$a-$b" }  "a".."d"    # "a", "a-b", "a-b-c", "a-b-c-d"
+
+=head2 any
+
+    my $bool = any { BLOCK } @list;
+
+I<Since version 1.33.>
+
+Similar to C<grep> in that it evaluates C<BLOCK> setting C<$_> to each element
+of C<@list> in turn. C<any> returns true if any element makes the C<BLOCK>
+return a true value. If C<BLOCK> never returns true or C<@list> was empty then
+it returns false.
+
+Many cases of using C<grep> in a conditional can be written using C<any>
+instead, as it can short-circuit after the first true result.
+
+    if( any { length > 10 } @strings ) {
+        # at least one string has more than 10 characters
+    }
+
+Note: Due to XS issues the block passed may be able to access the outer @_
+directly. This is not intentional and will break under debugger.
+
+=head2 all
+
+    my $bool = all { BLOCK } @list;
+
+I<Since version 1.33.>
+
+Similar to L</any>, except that it requires all elements of the C<@list> to
+make the C<BLOCK> return true. If any element returns false, then it returns
+false. If the C<BLOCK> never returns false or the C<@list> was empty then it
+returns true.
+
+Note: Due to XS issues the block passed may be able to access the outer @_
+directly. This is not intentional and will break under debugger.
+
+=head2 none
+
+=head2 notall
+
+    my $bool = none { BLOCK } @list;
+
+    my $bool = notall { BLOCK } @list;
+
+I<Since version 1.33.>
+
+Similar to L</any> and L</all>, but with the return sense inverted. C<none>
+returns true only if no value in the C<@list> causes the C<BLOCK> to return
+true, and C<notall> returns true only if not all of the values do.
+
+Note: Due to XS issues the block passed may be able to access the outer @_
+directly. This is not intentional and will break under debugger.
+
+=head2 first
+
+    my $val = first { BLOCK } @list;
+
+Similar to C<grep> in that it evaluates C<BLOCK> setting C<$_> to each element
+of C<@list> in turn. C<first> returns the first element where the result from
+C<BLOCK> is a true value. If C<BLOCK> never returns true or C<@list> was empty
+then C<undef> is returned.
 
     $foo = first { defined($_) } @list    # first defined value in @list
     $foo = first { $_ > $value } @list    # first value in @list which
                                           # is greater than $value
 
-This function could be implemented using C<reduce> like this
+=head2 max
 
-    $foo = reduce { defined($a) ? $a : wanted($b) ? $b : undef } undef, @list
+    my $num = max @list;
 
-for example wanted() could be defined() which would return the first
-defined value in @list
-
-=head2 max LIST
-
-Returns the entry in the list with the highest numerical value. If the
-list is empty then C<undef> is returned.
+Returns the entry in the list with the highest numerical value. If the list is
+empty then C<undef> is returned.
 
     $foo = max 1..10                # 10
     $foo = max 3,9,12               # 12
     $foo = max @bar, @baz           # whatever
 
-This function could be implemented using C<reduce> like this
+=head2 maxstr
 
-    $foo = reduce { $a > $b ? $a : $b } 1..10
+    my $str = maxstr @list;
 
-=head2 maxstr LIST
-
-Similar to C<max>, but treats all the entries in the list as strings
-and returns the highest string as defined by the C<gt> operator.
-If the list is empty then C<undef> is returned.
+Similar to L</max>, but treats all the entries in the list as strings and
+returns the highest string as defined by the C<gt> operator. If the list is
+empty then C<undef> is returned.
 
     $foo = maxstr 'A'..'Z'          # 'Z'
     $foo = maxstr "hello","world"   # "world"
     $foo = maxstr @bar, @baz        # whatever
 
-This function could be implemented using C<reduce> like this
+=head2 min
 
-    $foo = reduce { $a gt $b ? $a : $b } 'A'..'Z'
+    my $num = min @list;
 
-=head2 min LIST
-
-Similar to C<max> but returns the entry in the list with the lowest
-numerical value. If the list is empty then C<undef> is returned.
+Similar to L</max> but returns the entry in the list with the lowest numerical
+value. If the list is empty then C<undef> is returned.
 
     $foo = min 1..10                # 1
     $foo = min 3,9,12               # 3
     $foo = min @bar, @baz           # whatever
 
-This function could be implemented using C<reduce> like this
+=head2 minstr
 
-    $foo = reduce { $a < $b ? $a : $b } 1..10
+    my $str = minstr @list;
 
-=head2 minstr LIST
-
-Similar to C<min>, but treats all the entries in the list as strings
-and returns the lowest string as defined by the C<lt> operator.
-If the list is empty then C<undef> is returned.
+Similar to L</min>, but treats all the entries in the list as strings and
+returns the lowest string as defined by the C<lt> operator. If the list is
+empty then C<undef> is returned.
 
     $foo = minstr 'A'..'Z'          # 'A'
     $foo = minstr "hello","world"   # "hello"
     $foo = minstr @bar, @baz        # whatever
 
-This function could be implemented using C<reduce> like this
+=head2 product
 
-    $foo = reduce { $a lt $b ? $a : $b } 'A'..'Z'
+    my $num = product @list;
 
-=head2 sum LIST
+I<Since version 1.35.>
 
-Returns the sum of all the elements in LIST. If LIST is empty then
-C<undef> is returned.
+Returns the numerical product of all the elements in C<@list>. If C<@list> is
+empty then C<1> is returned.
+
+    $foo = product 1..10            # 3628800
+    $foo = product 3,9,12           # 324
+
+=head2 sum
+
+    my $num_or_undef = sum @list;
+
+Returns the numerical sum of all the elements in C<@list>. For backwards
+compatibility, if C<@list> is empty then C<undef> is returned.
 
     $foo = sum 1..10                # 55
     $foo = sum 3,9,12               # 24
     $foo = sum @bar, @baz           # whatever
 
-This function could be implemented using C<reduce> like this
+=head2 sum0
 
-    $foo = reduce { $a + $b } 1..10
+    my $num = sum0 @list;
 
-=head2 sum0 LIST
+I<Since version 1.26.>
 
-Similar to C<sum>, except this returns 0 when given an empty list, rather
+Similar to L</sum>, except this returns 0 when given an empty list, rather
 than C<undef>.
-
 =head1 KEY/VALUE PAIR LIST FUNCTIONS
 
-The following set of functions, all inspired by L<List::Pairwise>, consume
-an even-sized list of pairs. The pairs may be key/value associations from a
-hash, or just a list of values. The functions will all preserve the original
-ordering of the pairs, and will not be confused by multiple pairs having the
-same "key" value - nor even do they require that the first of each pair be a
-plain string.
+The following set of functions, all inspired by L<List::Pairwise>, consume an
+even-sized list of pairs. The pairs may be key/value associations from a hash,
+or just a list of values. The functions will all preserve the original ordering
+of the pairs, and will not be confused by multiple pairs having the same "key"
+value - nor even do they require that the first of each pair be a plain string.
 
-=head2 pairgrep BLOCK KVLIST
+B<NOTE>: At the time of writing, the following C<pair*> functions that take a
+block do not modify the value of C<$_> within the block, and instead operate
+using the C<$a> and C<$b> globals instead. This has turned out to be a poor
+design, as it precludes the ability to provide a C<pairsort> function. Better
+would be to pass pair-like objects as 2-element array references in C<$_>, in
+a style similar to the return value of the C<pairs> function. At some future
+version this behaviour may be added.
+
+Until then, users are alerted B<NOT> to rely on the value of C<$_> remaining
+unmodified between the outside and the inside of the control block. In
+particular, the following example is B<UNSAFE>:
+
+ my @kvlist = ...
+
+ foreach (qw( some keys here )) {
+    my @items = pairgrep { $a eq $_ } @kvlist;
+    ...
+ }
+
+Instead, write this using a lexical variable:
+
+ foreach my $key (qw( some keys here )) {
+    my @items = pairgrep { $a eq $key } @kvlist;
+    ...
+ }
+=head2 pairs
+
+    my @pairs = pairs @kvlist;
+
+I<Since version 1.29.>
+
+A convenient shortcut to operating on even-sized lists of pairs, this function
+returns a list of C<ARRAY> references, each containing two items from the
+given list. It is a more efficient version of
+
+    @pairs = pairmap { [ $a, $b ] } @kvlist
+
+It is most convenient to use in a C<foreach> loop, for example:
+
+    foreach my $pair ( pairs @kvlist ) {
+       my ( $key, $value ) = @$pair;
+       ...
+    }
+
+Since version C<1.39> these C<ARRAY> references are blessed objects,
+recognising the two methods C<key> and C<value>. The following code is
+equivalent:
+
+    foreach my $pair ( pairs @kvlist ) {
+       my $key   = $pair->key;
+       my $value = $pair->value;
+       ...
+    }
+
+Since version C<1.51> they also have a C<TO_JSON> method to ease
+serialisation.
+
+=head2 unpairs
+
+    my @kvlist = unpairs @pairs
+
+I<Since version 1.42.>
+
+The inverse function to C<pairs>; this function takes a list of C<ARRAY>
+references containing two elements each, and returns a flattened list of the
+two values from each of the pairs, in order. This is notionally equivalent to
+
+    my @kvlist = map { @{$_}[0,1] } @pairs
+
+except that it is implemented more efficiently internally. Specifically, for
+any input item it will extract exactly two values for the output list; using
+C<undef> if the input array references are short.
+
+Between C<pairs> and C<unpairs>, a higher-order list function can be used to
+operate on the pairs as single scalars; such as the following near-equivalents
+of the other C<pair*> higher-order functions:
+
+    @kvlist = unpairs grep { FUNC } pairs @kvlist
+    # Like pairgrep, but takes $_ instead of $a and $b
+
+    @kvlist = unpairs map { FUNC } pairs @kvlist
+    # Like pairmap, but takes $_ instead of $a and $b
+
+Note however that these versions will not behave as nicely in scalar context.
+
+Finally, this technique can be used to implement a sort on a keyvalue pair
+list; e.g.:
+
+    @kvlist = unpairs sort { $a->key cmp $b->key } pairs @kvlist
+
+=head2 pairkeys
+
+    my @keys = pairkeys @kvlist;
+
+I<Since version 1.29.>
+
+A convenient shortcut to operating on even-sized lists of pairs, this function
+returns a list of the the first values of each of the pairs in the given list.
+It is a more efficient version of
+
+    @keys = pairmap { $a } @kvlist
+
+=head2 pairvalues
+
+    my @values = pairvalues @kvlist;
+
+I<Since version 1.29.>
+
+A convenient shortcut to operating on even-sized lists of pairs, this function
+returns a list of the the second values of each of the pairs in the given list.
+It is a more efficient version of
+
+    @values = pairmap { $b } @kvlist
+
+=head2 pairgrep
+
+    my @kvlist = pairgrep { BLOCK } @kvlist;
+
+    my $count = pairgrep { BLOCK } @kvlist;
+
+I<Since version 1.29.>
 
 Similar to perl's C<grep> keyword, but interprets the given list as an
-even-sized list of pairs. It invokes the BLOCK multiple times, in scalar
+even-sized list of pairs. It invokes the C<BLOCK> multiple times, in scalar
 context, with C<$a> and C<$b> set to successive pairs of values from the
-KVLIST.
+C<@kvlist>.
 
-Returns an even-sized list of those pairs for which the BLOCK returned true
+Returns an even-sized list of those pairs for which the C<BLOCK> returned true
 in list context, or the count of the B<number of pairs> in scalar context.
-(Note, therefore, in scalar context that it returns a number half the size
-of the count of items it would have returned in list context).
+(Note, therefore, in scalar context that it returns a number half the size of
+the count of items it would have returned in list context).
 
     @subset = pairgrep { $a =~ m/^[[:upper:]]+$/ } @kvlist
 
-Similar to C<grep>, C<pairgrep> aliases C<$a> and C<$b> to elements of the
-given list. Any modifications of it by the code block will be visible to
-the caller.
+As with C<grep> aliasing C<$_> to list elements, C<pairgrep> aliases C<$a> and
+C<$b> to elements of the given list. Any modifications of it by the code block
+will be visible to the caller.
 
-=head2 pairfirst BLOCK KVLIST
+=head2 pairfirst
 
-Similar to the C<first> function, but interprets the given list as an
-even-sized list of pairs. It invokes the BLOCK multiple times, in scalar
+    my ( $key, $val ) = pairfirst { BLOCK } @kvlist;
+
+    my $found = pairfirst { BLOCK } @kvlist;
+
+I<Since version 1.30.>
+
+Similar to the L</first> function, but interprets the given list as an
+even-sized list of pairs. It invokes the C<BLOCK> multiple times, in scalar
 context, with C<$a> and C<$b> set to successive pairs of values from the
-KVLIST.
+C<@kvlist>.
 
-Returns the first pair of values from the list for which the BLOCK returned
+Returns the first pair of values from the list for which the C<BLOCK> returned
 true in list context, or an empty list of no such pair was found. In scalar
 context it returns a simple boolean value, rather than either the key or the
 value found.
 
     ( $key, $value ) = pairfirst { $a =~ m/^[[:upper:]]+$/ } @kvlist
 
-Similar to C<grep>, C<pairfirst> aliases C<$a> and C<$b> to elements of the
-given list. Any modifications of it by the code block will be visible to
-the caller.
+As with C<grep> aliasing C<$_> to list elements, C<pairfirst> aliases C<$a> and
+C<$b> to elements of the given list. Any modifications of it by the code block
+will be visible to the caller.
 
-=head2 pairmap BLOCK KVLIST
+=head2 pairmap
+
+    my @list = pairmap { BLOCK } @kvlist;
+
+    my $count = pairmap { BLOCK } @kvlist;
+
+I<Since version 1.29.>
 
 Similar to perl's C<map> keyword, but interprets the given list as an
-even-sized list of pairs. It invokes the BLOCK multiple times, in list
+even-sized list of pairs. It invokes the C<BLOCK> multiple times, in list
 context, with C<$a> and C<$b> set to successive pairs of values from the
-KVLIST.
+C<@kvlist>.
 
-Returns the concatenation of all the values returned by the BLOCK in list
-context, or the count of the number of items that would have been returned
-in scalar context.
+Returns the concatenation of all the values returned by the C<BLOCK> in list
+context, or the count of the number of items that would have been returned in
+scalar context.
 
     @result = pairmap { "The key $a has value $b" } @kvlist
 
-Similar to C<map>, C<pairmap> aliases C<$a> and C<$b> to elements of the
-given list. Any modifications of it by the code block will be visible to
-the caller.
-
-=head2 pairs KVLIST
-
-A convenient shortcut to operating on even-sized lists of pairs, this
-function returns a list of ARRAY references, each containing two items from
-the given list. It is a more efficient version of
-
-    pairmap { [ $a, $b ] } KVLIST
-
-It is most convenient to use in a C<foreach> loop, for example:
-
-    foreach ( pairs @KVLIST ) {
-       my ( $key, $value ) = @$_;
-       ...
-    }
-
-=head2 pairkeys KVLIST
-
-A convenient shortcut to operating on even-sized lists of pairs, this
-function returns a list of the the first values of each of the pairs in
-the given list. It is a more efficient version of
-
-    pairmap { $a } KVLIST
-
-=head2 pairvalues KVLIST
-
-A convenient shortcut to operating on even-sized lists of pairs, this
-function returns a list of the the second values of each of the pairs in
-the given list. It is a more efficient version of
-
-    pairmap { $b } KVLIST
+As with C<map> aliasing C<$_> to list elements, C<pairmap> aliases C<$a> and
+C<$b> to elements of the given list. Any modifications of it by the code block
+will be visible to the caller.
 
+See L</KNOWN BUGS> for a known-bug with C<pairmap>, and a workaround.
 =head1 OTHER FUNCTIONS
+=head2 shuffle
 
-=head2 shuffle LIST
+    my @values = shuffle @values;
 
-Returns the elements of LIST in a random order
+Returns the values of the input in a random order
 
     @cards = shuffle 0..51      # 0..51 in a random order
 
+This function is affected by the C<$RAND> variable.
+=head2 sample
+
+    my @items = sample $count, @values
+
+I<Since version 1.54.>
+
+Randomly select the given number of elements from the input list. Any given
+position in the input list will be selected at most once.
+
+If there are fewer than C<$count> items in the list then the function will
+return once all of them have been randomly selected; effectively the function
+behaves similarly to L</shuffle>.
+
+This function is affected by the C<$RAND> variable.
+
+=head2 uniq
+
+    my @subset = uniq @values
+
+I<Since version 1.45.>
+
+Filters a list of values to remove subsequent duplicates, as judged by a
+DWIM-ish string equality or C<undef> test. Preserves the order of unique
+elements, and retains the first value of any duplicate set.
+
+    my $count = uniq @values
+
+In scalar context, returns the number of elements that would have been
+returned as a list.
+
+The C<undef> value is treated by this function as distinct from the empty
+string, and no warning will be produced. It is left as-is in the returned
+list. Subsequent C<undef> values are still considered identical to the first,
+and will be removed.
+
+=head2 uniqnum
+
+    my @subset = uniqnum @values
+
+I<Since version 1.44.>
+
+Filters a list of values to remove subsequent duplicates, as judged by a
+numerical equality test. Preserves the order of unique elements, and retains
+the first value of any duplicate set.
+
+    my $count = uniqnum @values
+
+In scalar context, returns the number of elements that would have been
+returned as a list.
+
+Note that C<undef> is treated much as other numerical operations treat it; it
+compares equal to zero but additionally produces a warning if such warnings
+are enabled (C<use warnings 'uninitialized';>). In addition, an C<undef> in
+the returned list is coerced into a numerical zero, so that the entire list of
+values returned by C<uniqnum> are well-behaved as numbers.
+
+Note also that multiple IEEE C<NaN> values are treated as duplicates of
+each other, regardless of any differences in their payloads, and despite
+the fact that C<< 0+'NaN' == 0+'NaN' >> yields false.
+
+=head2 uniqstr
+
+    my @subset = uniqstr @values
+
+I<Since version 1.45.>
+
+Filters a list of values to remove subsequent duplicates, as judged by a
+string equality test. Preserves the order of unique elements, and retains the
+first value of any duplicate set.
+
+    my $count = uniqstr @values
+
+In scalar context, returns the number of elements that would have been
+returned as a list.
+
+Note that C<undef> is treated much as other string operations treat it; it
+compares equal to the empty string but additionally produces a warning if such
+warnings are enabled (C<use warnings 'uninitialized';>). In addition, an
+C<undef> in the returned list is coerced into an empty string, so that the
+entire list of values returned by C<uniqstr> are well-behaved as strings.
+=head2 head
+
+    my @values = head $size, @list;
+
+I<Since version 1.50.>
+
+Returns the first C<$size> elements from C<@list>. If C<$size> is negative, returns
+all but the last C<$size> elements from C<@list>.
+
+    @result = head 2, qw( foo bar baz );
+    # foo, bar
+
+    @result = head -2, qw( foo bar baz );
+    # foo
+
+=head2 tail
+
+    my @values = tail $size, @list;
+
+I<Since version 1.50.>
+
+Returns the last C<$size> elements from C<@list>. If C<$size> is negative, returns
+all but the first C<$size> elements from C<@list>.
+
+    @result = tail 2, qw( foo bar baz );
+    # bar, baz
+
+    @result = tail -2, qw( foo bar baz );
+    # baz
+
 =head1 List::SomeUtils FUNCTIONS
 
 =head2 Junctions
@@ -438,10 +756,13 @@ Evaluation of BLOCK will immediately stop at the second true value.
 
 =head3 apply BLOCK LIST
 
-Applies BLOCK to each item in LIST and returns a list of the values after BLOCK
-has been applied. In scalar context, the last element is returned.  This
-function is similar to C<map> but will not modify the elements of the input
-list:
+Makes a copy of the list and then passes each element I<from the copy> to the
+BLOCK. Any changes or assignments to C<$_> in the BLOCK will only affect the
+elements of the new list. However, if C<$_> is a reference then changes to the
+referenced value will be seen in both the original and new list.
+
+This function is similar to C<map> but will not modify the elements of the
+input list:
 
   my @list = (1 .. 4);
   my @mult = apply { $_ *= 2 } @list;
@@ -455,6 +776,15 @@ Think of it as syntactic sugar for
 
   for (my @mult = @list) { $_ *= 2 }
 
+Note that you must alter C<$_> directly inside BLOCK in order for changes to
+make effect. New value returned from the BLOCK are ignored:
+
+  # @new is identical to @list.
+  my @new = apply { $_ * 2 } @list;
+
+  # @new is different from @list
+  my @new = apply { $_ =* 2 } @list;
+
 =head3 insert_after BLOCK VALUE LIST
 
 Inserts VALUE after the first item in LIST for which the criterion in BLOCK is
@@ -591,7 +921,7 @@ Negative values are only ok when they refer to a partition previously created:
 
   my @idx  = ( 0, 1, -1 );
   my $i    = 0;
-  my @part = part { $idx[$++ % 3] } 1 .. 8; # [1, 4, 7], [2, 3, 5, 6, 8]
+  my @part = part { $idx[$i++ % 3] } 1 .. 8; # [1, 4, 7], [2, 3, 5, 6, 8]
 
 =head2 Iteration
 
@@ -851,10 +1181,49 @@ If more than one item appears the same number of times in the list, all such
 items will be returned. For example, the mode of a unique list is the list
 itself.
 
-This function B<always> returns a list. That means that in scalar context you
-get a count indicating the number of modes in the list.
+This function returns a list in list context. In scalar context it returns a
+count indicating the number of modes in the list.
 
-=head1 List::UtilsBy Functions
+=head1 List::UtilsBy FUNCTIONS
+
+All functions added since version 0.04 unless otherwise stated, as the
+original names for earlier versions were renamed.
+
+=head2 sort_by
+
+   @vals = sort_by { KEYFUNC } @vals
+
+Returns the list of values sorted according to the string values returned by
+the C<KEYFUNC> block or function. A typical use of this may be to sort objects
+according to the string value of some accessor, such as
+
+   sort_by { $_->name } @people
+
+The key function is called in scalar context, being passed each value in turn
+as both C<$_> and the only argument in the parameters, C<@_>. The values are
+then sorted according to string comparisons on the values returned.
+
+This is equivalent to
+
+   sort { $a->name cmp $b->name } @people
+
+except that it guarantees the C<name> accessor will be executed only once per
+value.
+
+One interesting use-case is to sort strings which may have numbers embedded in
+them "naturally", rather than lexically.
+
+   sort_by { s/(\d+)/sprintf "%09d", $1/eg; $_ } @strings
+
+This sorts strings by generating sort keys which zero-pad the embedded numbers
+to some level (9 digits in this case), helping to ensure the lexical sort puts
+them in the correct order.
+
+=head2 nsort_by
+
+   @vals = nsort_by { KEYFUNC } @vals
+
+Similar to L</sort_by> but compares its key values numerically.
 
 =head2 rev_sort_by
 
@@ -910,6 +1279,8 @@ result from the key function. Also provided as C<nmin_by>
 
    ( $minimal, $maximal ) = minmax_by { KEYFUNC } @vals
 
+I<Since version 0.11.>
+
 Similar to calling both L</min_by> and L</max_by> with the same key function
 on the same list. This version is more efficient than calling the two other
 functions individually, as it has less work to perform overall. In the case of
@@ -956,6 +1327,8 @@ manner).
 
    %counts = count_by { KEYFUNC } @vals
 
+I<Since version 0.07.>
+
 Returns a key/value list of integers, giving the number of times the key
 function block returned the key, for each value in the list.
 
@@ -1002,6 +1375,8 @@ Haskell, but that name would not fit the naming scheme used by this module).
 
    $arr0, $arr1, $arr2, ... = unzip_by { ITEMFUNC } @vals
 
+I<Since version 0.09.>
+
 Returns a list of ARRAY references containing the values returned by the
 function block, when invoked for each of the values given in the input list.
 Each of the returned ARRAY references will contain the values returned at that
@@ -1022,6 +1397,8 @@ function.
 
    @vals = extract_by { SELECTFUNC } @arr
 
+I<Since version 0.05.>
+
 Removes elements from the referenced array on which the selection function
 returns true, and returns a list containing those elements. This function is
 similar to C<grep>, except that it modifies the referenced array to remove the
@@ -1050,6 +1427,8 @@ will strengthen them all again.
 
    $val = extract_first_by { SELECTFUNC } @arr
 
+I<Since version 0.10.>
+
 A hybrid between L</extract_by> and C<List::Util::first>. Removes the first
 element from the referenced array on which the selection function returns
 true, returning it.
@@ -1066,15 +1445,19 @@ no matching element, and the first matching element being C<undef>.
 
    @vals = weighted_shuffle_by { WEIGHTFUNC } @vals
 
+I<Since version 0.07.>
+
 Returns the list of values shuffled into a random order. The randomisation is
 not uniform, but weighted by the value returned by the C<WEIGHTFUNC>. The
-probability of each item being returned first will be distributed with the
+probabilty of each item being returned first will be distributed with the
 distribution of the weights, and so on recursively for the remaining items.
 
 =head2 bundle_by
 
    @vals = bundle_by { BLOCKFUNC } $number, @vals
 
+I<Since version 0.07.>
+
 Similar to a regular C<map> functional, returns a list of the values returned
 by C<BLOCKFUNC>. Values from the input list are given to the block function in
 bundles of C<$number>.
@@ -1132,12 +1515,20 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Karen Etheridge Ricardo Signes Yanick Champoux
+=for stopwords Andy Jack Dave Jacoby Karen Etheridge Ricardo Signes Yanick Champoux
 
 =over 4
 
 =item *
 
+Andy Jack <github@veracity.ca>
+
+=item *
+
+Dave Jacoby <jacoby.david@gmail.com>
+
+=item *
+
 Karen Etheridge <ether@cpan.org>
 
 =item *
@@ -1152,7 +1543,7 @@ Yanick Champoux <yanick@babyl.dyndns.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by Dave Rolsky.
+This software is Copyright (c) 2020 by Dave Rolsky.
 
 This is free software, licensed under:
 
